@@ -1,5 +1,5 @@
 /* Basic FTP routines.
-   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
 
 This file is part of Wget.
 
@@ -42,13 +42,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "connect.h"
 #include "host.h"
 
-#ifndef errno
-extern int errno;
-#endif
-#ifndef h_errno
-extern int h_errno;
-#endif
-
 char ftp_last_respline[128];
 
 
@@ -185,7 +178,7 @@ ftp_login (struct rbuf *rbuf, const char *acc, const char *pass)
 
     for (i = 0; i < ARRAY_SIZE (skey_head); i++)
       {
-	if (strncmp (skey_head[i], respline, strlen (skey_head[i])) == 0)
+	if (strncasecmp (skey_head[i], respline, strlen (skey_head[i])) == 0)
 	  break;
       }
     if (i < ARRAY_SIZE (skey_head))
