@@ -1,5 +1,5 @@
 /* File Transfer Protocol support.
-   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
 
 This file is part of Wget.
 
@@ -33,6 +33,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <sys/types.h>
 #include <assert.h>
 #include <errno.h>
+#ifndef WINDOWS
+# include <netdb.h>		/* for h_errno */
+#endif
 
 #include "wget.h"
 #include "utils.h"
@@ -50,7 +53,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 extern int errno;
 #endif
 #ifndef h_errno
+# ifndef __CYGWIN__
 extern int h_errno;
+# endif
 #endif
 
 /* File where the "ls -al" listing will be saved.  */
@@ -90,7 +95,7 @@ ftp_expected_bytes (const char *s)
 	++s;
       if (!*s)
 	return 0;
-      if (tolower (*s) != 'b')
+      if (TOLOWER (*s) != 'b')
 	continue;
       if (strncasecmp (s, "byte", 4))
 	continue;
@@ -130,7 +135,7 @@ getftp (const struct urlinfo *u, long *len, long restval, ccon *con)
   search_netrc (u->host, (const char **)&user, (const char **)&passwd, 1);
   user = user ? user : opt.ftp_acc;
   if (!opt.ftp_pass)
-    opt.ftp_pass = xstrdup (ftp_getaddress ());
+    opt.ftp_pass = ftp_getaddress ();
   passwd = passwd ? passwd : opt.ftp_pass;
   assert (user && passwd);
 
@@ -243,8 +248,8 @@ Error in server response, closing control connection.\n"));
 	}
       /* Third: Set type to Image (binary).  */
       if (!opt.server_response)
-	logprintf (LOG_VERBOSE, "==> TYPE %c ... ", toupper (u->ftp_type));
-      err = ftp_type (&con->rbuf, toupper (u->ftp_type));
+	logprintf (LOG_VERBOSE, "==> TYPE %c ... ", TOUPPER (u->ftp_type));
+      err = ftp_type (&con->rbuf, TOUPPER (u->ftp_type));
       /* FTPRERR, WRITEFAILED, FTPUNKNOWNTYPE */
       switch (err)
 	{
@@ -268,7 +273,7 @@ Error in server response, closing control connection.\n"));
 	  logputs (LOG_VERBOSE, "\n");
 	  logprintf (LOG_NOTQUIET,
 		     _("Unknown type `%c', closing control connection.\n"),
-		     toupper (u->ftp_type));
+		     TOUPPER (u->ftp_type));
 	  CLOSE (csock);
 	  rbuf_uninitialize (&con->rbuf);
 	  return err;
@@ -337,7 +342,7 @@ Error in server response, closing control connection.\n"));
   /* If anything is to be retrieved, PORT (or PASV) must be sent.  */
   if (cmd & (DO_LIST | DO_RETR))
     {
-      if (opt.ftp_pasv)
+      if (opt.ftp_pasv > 0)
 	{
 	  char thost[256];
 	  unsigned short tport;
@@ -648,6 +653,15 @@ Error in server response, closing control connection.\n"));
       expected_bytes = ftp_expected_bytes (ftp_last_respline);
     } /* cmd & DO_LIST */
 
+  /* Some FTP servers return the total length of file after REST
+     command, others just return the remaining size. */
+  if (*len && restval && expected_bytes
+      && (expected_bytes == *len - restval))
+    {
+      DEBUGP (("Lying FTP server found, adjusting.\n"));
+      expected_bytes = *len;
+    }
+
   /* If no transmission was required, then everything is OK.  */
   if (!(cmd & (DO_LIST | DO_RETR)))
     return RETRFINISHED;
@@ -685,7 +699,16 @@ Error in server response, closing control connection.\n"));
 	}
     }
   else
-    fp = opt.dfp;
+    {
+      fp = opt.dfp;
+      if (!restval)
+	{
+	  /* This will silently fail for streams that don't correspond
+	     to regular files, but that's OK.  */
+	  rewind (fp);
+	  clearerr (fp);
+	}
+    }
 
   if (*len)
     {
@@ -711,10 +734,18 @@ Error in server response, closing control connection.\n"));
   /* Close data connection socket.  */
   closeport (dtsock);
   /* Close the local file.  */
-  if (!opt.dfp || con->cmd & DO_LIST)
-    fclose (fp);
-  else
-    fflush (fp);
+  {
+    /* Close or flush the file.  We have to be careful to check for
+       error here.  Checking the result of fwrite() is not enough --
+       errors could go unnoticed!  */
+    int flush_res;
+    if (!opt.dfp || con->cmd & DO_LIST)
+      flush_res = fclose (fp);
+    else
+      flush_res = fflush (fp);
+    if (flush_res == EOF)
+      res = -2;
+  }
   /* If get_contents couldn't write to fp, bail out.  */
   if (res == -2)
     {
@@ -813,8 +844,6 @@ Error in server response, closing control connection.\n"));
 static uerr_t
 ftp_loop_internal (struct urlinfo *u, struct fileinfo *f, ccon *con)
 {
-  static int first_retrieval = 1;
-
   int count, orig_lp;
   long restval, len;
   char *tms, *tmrate, *locf;
@@ -851,12 +880,7 @@ ftp_loop_internal (struct urlinfo *u, struct fileinfo *f, ccon *con)
     {
       /* Increment the pass counter.  */
       ++count;
-      /* Wait before the retrieval (unless this is the very first
-	 retrieval).  */
-      if (!first_retrieval && opt.wait)
-	sleep (opt.wait);
-      if (first_retrieval)
-	first_retrieval = 0;
+      sleep_between_retrievals (count);
       if (con->st & ON_YOUR_OWN)
 	{
 	  con->cmd = 0;
@@ -947,6 +971,11 @@ ftp_loop_internal (struct urlinfo *u, struct fileinfo *f, ccon *con)
 	  /* Not as great.  */
 	  abort ();
 	}
+
+      /* If we get out of the switch above without continue'ing, we've
+	 successfully downloaded a file.  Remember this fact. */
+      downloaded_file(FILE_DOWNLOADED_NORMALLY, locf);
+
       if (con->st & ON_YOUR_OWN)
 	{
 	  CLOSE (RBUF_FD (&con->rbuf));
@@ -954,15 +983,52 @@ ftp_loop_internal (struct urlinfo *u, struct fileinfo *f, ccon *con)
 	}
       logprintf (LOG_VERBOSE, _("%s (%s) - `%s' saved [%ld]\n\n"),
 		 tms, tmrate, locf, len);
-      logprintf (LOG_NONVERBOSE, "%s URL: %s [%ld] -> \"%s\" [%d]\n",
-		 tms, u->url, len, locf, count);
-      /* Do not count listings among the downloaded stuff, since they
-	 will get deleted anyway.  */
-      if (!(con->cmd & DO_LIST))
+      if (!opt.verbose && !opt.quiet)
 	{
-	  ++opt.numurls;
-	  opt.downloaded += len;
+	  /* Need to hide the password from the URL.  The `if' is here
+             so that we don't do the needless allocation every
+             time. */
+	  char *hurl = str_url (u->proxy ? u->proxy : u, 1);
+	  logprintf (LOG_NONVERBOSE, "%s URL: %s [%ld] -> \"%s\" [%d]\n",
+		     tms, hurl, len, locf, count);
+	  free (hurl);
 	}
+
+      if ((con->cmd & DO_LIST))
+	/* This is a directory listing file. */
+	{
+	  if (!opt.remove_listing)
+	    /* --dont-remove-listing was specified, so do count this towards the
+	       number of bytes and files downloaded. */
+	    {
+	      downloaded_increase (len);
+	      opt.numurls++;
+	    }
+
+	  /* Deletion of listing files is not controlled by --delete-after, but
+	     by the more specific option --dont-remove-listing, and the code
+	     to do this deletion is in another function. */
+	}
+      else
+	/* This is not a directory listing file. */
+	{
+	  /* Unlike directory listing files, don't pretend normal files weren't
+	     downloaded if they're going to be deleted.  People seeding proxies,
+	     for instance, may want to know how many bytes and files they've
+	     downloaded through it. */
+	  downloaded_increase (len);
+	  opt.numurls++;
+
+	  if (opt.delete_after)
+	    {
+	      DEBUGP (("Removing file due to --delete-after in"
+		       " ftp_loop_internal():\n"));
+	      logprintf (LOG_VERBOSE, _("Removing %s.\n"), locf);
+	      if (unlink (locf))
+		logprintf (LOG_NOTQUIET, "unlink: %s\n", strerror (errno));
+	    }
+	}
+      
       /* Restore the original leave-pendingness.  */
       if (orig_lp)
 	con->cmd |= LEAVE_PENDING;
@@ -1044,7 +1110,7 @@ ftp_retrieve_list (struct urlinfo *u, struct fileinfo *f, ccon *con)
 
   /* Increase the depth.  */
   ++depth;
-  if (opt.reclevel && depth > opt.reclevel)
+  if (opt.reclevel != INFINITE_RECURSION && depth > opt.reclevel)
     {
       DEBUGP ((_("Recursion depth %d exceeded max. depth %d.\n"),
 	       depth, opt.reclevel));
@@ -1071,7 +1137,7 @@ ftp_retrieve_list (struct urlinfo *u, struct fileinfo *f, ccon *con)
 
   while (f)
     {
-      if (opt.quota && opt.downloaded > opt.quota)
+      if (downloaded_exceeds_quota ())
 	{
 	  --depth;
 	  return QUOTEXC;
@@ -1086,6 +1152,11 @@ ftp_retrieve_list (struct urlinfo *u, struct fileinfo *f, ccon *con)
       if (opt.timestamping && f->type == FT_PLAINFILE)
 	{
 	  struct stat st;
+	  /* If conversion of HTML files retrieved via FTP is ever implemented,
+	     we'll need to stat() <file>.orig here when -K has been specified.
+	     I'm not implementing it now since files on an FTP server are much
+	     more likely than files on an HTTP server to legitimately have a
+	     .orig suffix. */
 	  if (!stat (u->local, &st))
 	    {
 	      /* Else, get it from the file.  */
@@ -1094,13 +1165,13 @@ ftp_retrieve_list (struct urlinfo *u, struct fileinfo *f, ccon *con)
 	      if (local_size == f->size && tml >= f->tstamp)
 		{
 		  logprintf (LOG_VERBOSE, _("\
-Local file `%s' is more recent, not retrieving.\n\n"), u->local);
+Server file no newer than local file `%s' -- not retrieving.\n\n"), u->local);
 		  dlthis = 0;
 		}
 	      else if (local_size != f->size)
 		{
 		  logprintf (LOG_VERBOSE, _("\
-The sizes do not match (local %ld), retrieving.\n"), local_size);
+The sizes do not match (local %ld) -- retrieving.\n"), local_size);
 		}
 	    }
 	}	/* opt.timestamping && f->type == FT_PLAINFILE */
@@ -1181,18 +1252,28 @@ Already have correct symlink %s -> %s\n\n"),
       /* Set the time-stamp information to the local file.  Symlinks
 	 are not to be stamped because it sets the stamp on the
 	 original.  :( */
-      if (!opt.dfp
-	  && !(f->type == FT_SYMLINK && !opt.retr_symlinks)
+      if (!(f->type == FT_SYMLINK && !opt.retr_symlinks)
 	  && f->tstamp != -1
           && dlthis
 	  && file_exists_p (u->local))
 	{
-	  touch (u->local, f->tstamp);
+	  /* #### This code repeats in http.c and ftp.c.  Move it to a
+             function!  */
+	  const char *fl = NULL;
+	  if (opt.output_document)
+	    {
+	      if (opt.od_known_regular)
+		fl = opt.output_document;
+	    }
+	  else
+	    fl = u->local;
+	  if (fl)
+	    touch (fl, f->tstamp);
 	}
       else if (f->tstamp == -1)
 	logprintf (LOG_NOTQUIET, _("%s: corrupt time-stamp.\n"), u->local);
 
-      if (f->perms && dlthis)
+      if (f->perms && f->type == FT_PLAINFILE && dlthis)
 	chmod (u->local, f->perms);
       else
 	DEBUGP (("Unrecognized permissions for %s.\n", u->local));
@@ -1207,7 +1288,8 @@ Already have correct symlink %s -> %s\n\n"),
       f = f->next;
     } /* while */
   /* We do not want to call ftp_retrieve_dirs here */
-  if (opt.recursive && !(opt.reclevel && depth >= opt.reclevel))
+  if (opt.recursive &&
+      !(opt.reclevel != INFINITE_RECURSION && depth >= opt.reclevel))
     err = ftp_retrieve_dirs (u, orig, con);
   else if (opt.recursive)
     DEBUGP ((_("Will not retrieve dirs since depth is %d (max %d).\n"),
@@ -1231,7 +1313,7 @@ ftp_retrieve_dirs (struct urlinfo *u, struct fileinfo *f, ccon *con)
     {
       int len;
 
-      if (opt.quota && opt.downloaded > opt.quota)
+      if (downloaded_exceeds_quota ())
 	break;
       if (f->type != FT_DIRECTORY)
 	continue;
@@ -1352,7 +1434,7 @@ ftp_retrieve_glob (struct urlinfo *u, ccon *con, int action)
 	}
     }
   freefileinfo (start);
-  if (opt.quota && opt.downloaded > opt.quota)
+  if (downloaded_exceeds_quota ())
     return QUOTEXC;
   else
     /* #### Should we return `res' here?  */
