@@ -64,12 +64,18 @@ struct options
   char **domains;		/* See host.c */
   char **exclude_domains;
 
+  char **follow_tags;           /* List of HTML tags to recursively follow. */
+  char **ignore_tags;           /* List of HTML tags to ignore if recursing. */
+
   int follow_ftp;		/* Are FTP URL-s followed in recursive
 				   retrieving? */
   int retr_symlinks;		/* Whether we retrieve symlinks in
 				   FTP. */
   char *output_document;	/* The output file to which the
 				   documents will be printed.  */
+  int od_known_regular;		/* whether output_document is a
+                                   regular file we can manipulate,
+                                   i.e. not `-' or a device file. */
   FILE *dfp;			/* The file pointer to the output
 				   document. */
 
@@ -95,12 +101,14 @@ struct options
   long timeout;			/* The value of read timeout in
 				   seconds. */
 #endif
-  long wait;			/* The wait period between retries. */
+  long wait;			/* The wait period between retrievals. */
+  long waitretry;		/* The wait period between retries. - HEH */
   int use_robots;		/* Do we heed robots.txt? */
 
   long quota;			/* Maximum number of bytes to
 				   retrieve. */
-  long downloaded;		/* How much we downloaded already. */
+  VERY_LONG_TYPE downloaded;	/* How much we downloaded already. */
+  int downloaded_overflow;	/* Whether the above overflowed. */
   int numurls;			/* Number of successfully downloaded
 				   URLs */
 
@@ -113,11 +121,16 @@ struct options
 #endif /* DEBUG */
 
   int timestamping;		/* Whether to use time-stamping. */
-  int backups;			/* Are backups made? */
+
+  int backup_converted;		/* Do we save pre-converted files as *.orig? */
+  int backups;			/* Are numeric backups made? */
 
   char *useragent;		/* Naughty User-Agent, which can be
 				   set to something other than
 				   Wget. */
+  char *referer;		/* Naughty Referer, which can be
+				   set to something other than
+				   NULL. */
   int convert_links;		/* Will the links be converted
 				   locally? */
   int remove_listing;		/* Do we remove .listing files
@@ -132,6 +145,13 @@ struct options
 
   int delete_after;		/* Whether the files will be deleted
 				   after download. */
+
+  int html_extension;		/* Use ".html" extension on all text/html? */
+
+  int page_requisites;		/* Whether we need to download all files
+				   necessary to display a page properly. */
+
+  struct sockaddr_in *bind_address; /* What local IP address to bind to. */
 };
 
 #ifndef OPTIONS_DEFINED_HERE
