@@ -1,5 +1,5 @@
 /* Handling of recursive HTTP retrieving.
-   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 2000 Free Software Foundation, Inc.
 
 This file is part of Wget.
 
@@ -43,6 +43,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "fnmatch.h"
 #include "host.h"
 
+#ifndef errno
+extern int errno;
+#endif
+
 extern char *version_string;
 
 #define ROBOTS_FILENAME "robots.txt"
@@ -106,7 +110,7 @@ recursive_reset (void)
 }
 
 /* The core of recursive retrieving.  Endless recursion is avoided by
-   having all URL-s stored to a linked list of URL-s, which is checked
+   having all URLs stored to a linked list of URLs, which is checked
    before loading any URL.  That way no URL can get loaded twice.
 
    The function also supports specification of maximum recursion depth
@@ -116,7 +120,7 @@ recursive_retrieve (const char *file, const char *this_url)
 {
   char *constr, *filename, *newloc;
   char *canon_this_url = NULL;
-  int dt, inl;
+  int dt, inl, dash_p_leaf_HTML = FALSE;
   int this_url_ftp;            /* See below the explanation */
   uerr_t err;
   struct urlinfo *rurl;
@@ -127,7 +131,7 @@ recursive_retrieve (const char *file, const char *this_url)
   assert (this_url != NULL);
   assert (file != NULL);
   /* If quota was exceeded earlier, bail out.  */
-  if (opt.quota && (opt.downloaded > opt.quota))
+  if (downloaded_exceeds_quota ())
     return QUOTEXC;
   /* Cache the current URL in the list.  */
   if (first_time)
@@ -163,13 +167,24 @@ recursive_retrieve (const char *file, const char *this_url)
   else
     ++depth;
 
-  /* Bail out if opt.reclevel is exceeded.  */
-  if ((opt.reclevel != 0) && (depth > opt.reclevel))
+  if (opt.reclevel != INFINITE_RECURSION && depth > opt.reclevel)
+    /* We've exceeded the maximum recursion depth specified by the user. */
     {
-      DEBUGP (("Recursion depth %d exceeded max. depth %d.\n",
-	       depth, opt.reclevel));
-      --depth;
-      return RECLEVELEXC;
+      if (opt.page_requisites && depth <= opt.reclevel + 1)
+	/* When -p is specified, we can do one more partial recursion from the
+	   "leaf nodes" on the HTML document tree.  The recursion is partial in
+	   that we won't traverse any <A> or <AREA> tags, nor any <LINK> tags
+	   except for <LINK REL="stylesheet">. */
+	dash_p_leaf_HTML = TRUE;
+      else
+	/* Either -p wasn't specified or it was and we've already gone the one
+	   extra (pseudo-)level that it affords us, so we need to bail out. */
+	{
+	  DEBUGP (("Recursion depth %d exceeded max. depth %d.\n",
+		   depth, opt.reclevel));
+	  --depth;
+	  return RECLEVELEXC;
+	}
     }
 
   /* Determine whether this_url is an FTP URL.  If it is, it means
@@ -179,15 +194,15 @@ recursive_retrieve (const char *file, const char *this_url)
   this_url_ftp = (urlproto (this_url) == URLFTP);
 
   /* Get the URL-s from an HTML file: */
-  url_list = get_urls_html (file,
-			    canon_this_url ? canon_this_url : this_url, 0);
+  url_list = get_urls_html (file, canon_this_url ? canon_this_url : this_url,
+			    0, dash_p_leaf_HTML);
 
   /* Decide what to do with each of the URLs.  A URL will be loaded if
      it meets several requirements, discussed later.  */
   for (cur_url = url_list; cur_url; cur_url = cur_url->next)
     {
       /* If quota was exceeded earlier, bail out.  */
-      if (opt.quota && (opt.downloaded > opt.quota))
+      if (downloaded_exceeds_quota ())
 	break;
       /* Parse the URL for convenient use in other functions, as well
 	 as to get the optimized form.  It also checks URL integrity.  */
@@ -221,9 +236,9 @@ recursive_retrieve (const char *file, const char *this_url)
 	 Addendum: If the URL is FTP, and it is to be loaded, only the
 	 domain and suffix settings are "stronger".
 
-	 Note that .html and (yuck) .htm will get loaded
-	 regardless of suffix rules (but that is remedied later with
-	 unlink).
+	 Note that .html and (yuck) .htm will get loaded regardless of
+	 suffix rules (but that is remedied later with unlink) unless
+	 the depth equals the maximum depth.
 
 	 More time- and memory- consuming tests should be put later on
 	 the list.  */
@@ -306,11 +321,14 @@ recursive_retrieve (const char *file, const char *this_url)
 	     b) it is "htm"
 
 	     If the file *is* supposed to be HTML, it will *not* be
-	     subject to acc/rej rules.  That's why the `!'.  */
+            subject to acc/rej rules, unless a finite maximum depth has
+            been specified and the current depth is the maximum depth. */
 	  if (!
 	      (!*u->file
 	       || (((suf = suffix (constr)) != NULL)
-		   && (!strcmp (suf, "html") || !strcmp (suf, "htm")))))
+                  && ((!strcmp (suf, "html") || !strcmp (suf, "htm"))
+                      && ((opt.reclevel != INFINITE_RECURSION) &&
+			  (depth != opt.reclevel))))))
 	    {
 	      if (!acceptable (u->file))
 		{
@@ -333,7 +351,7 @@ recursive_retrieve (const char *file, const char *this_url)
 	      char *p;
 	      /* Just lowercase the hostname.  */
 	      for (p = u->host; *p; p++)
-		*p = tolower (*p);
+		*p = TOLOWER (*p);
 	      free (u->url);
 	      u->url = str_url (u, 0);
 	    }
@@ -441,10 +459,15 @@ recursive_retrieve (const char *file, const char *this_url)
 	  else
 	    DEBUGP (("%s is not text/html so we don't chase.\n",
 		     filename ? filename: "(null)"));
-	  /* If an suffix-rejected file was loaded only because it was HTML,
-	     undo the error now */
+
 	  if (opt.delete_after || (filename && !acceptable (filename)))
+	    /* Either --delete-after was specified, or we loaded this otherwise
+	       rejected (e.g. by -R) HTML file just so we could harvest its
+	       hyperlinks -- in either case, delete the local file. */
 	    {
+	      DEBUGP (("Removing file due to %s in recursive_retrieve():\n",
+		       opt.delete_after ? "--delete-after" :
+		       "recursive rejection criteria"));
 	      logprintf (LOG_VERBOSE,
 			 (opt.delete_after ? _("Removing %s.\n")
 			  : _("Removing %s since it should be rejected.\n")),
@@ -453,6 +476,7 @@ recursive_retrieve (const char *file, const char *this_url)
 		logprintf (LOG_NOTQUIET, "unlink: %s\n", strerror (errno));
 	      dt &= ~RETROKF;
 	    }
+
 	  /* If everything was OK, and links are to be converted, let's
 	     store the local filename.  */
 	  if (opt.convert_links && (dt & RETROKF) && (filename != NULL))
@@ -468,7 +492,7 @@ recursive_retrieve (const char *file, const char *this_url)
       freeurl (u, 1);
       /* Increment the pbuf for the appropriate size.  */
     }
-  if (opt.convert_links)
+  if (opt.convert_links && !opt.delete_after)
     convert_links (file, url_list);
   /* Free the linked list of URL-s.  */
   free_urlpos (url_list);
@@ -476,7 +500,7 @@ recursive_retrieve (const char *file, const char *this_url)
   FREE_MAYBE (canon_this_url);
   /* Decrement the recursion depth.  */
   --depth;
-  if (opt.quota && (opt.downloaded > opt.quota))
+  if (downloaded_exceeds_quota ())
     return QUOTEXC;
   else
     return RETROK;
@@ -502,7 +526,7 @@ recursive_retrieve (const char *file, const char *this_url)
 
    This is why Wget must, after the whole retrieval, call
    convert_all_links to go once more through the entire list of
-   retrieved HTML-s, and re-convert them.
+   retrieved HTMLs, and re-convert them.
 
    All the downloaded HTMLs are kept in urls_html, and downloaded URLs
    in urls_downloaded.  From these two lists information is
@@ -529,7 +553,8 @@ convert_all_links (void)
       else
 	DEBUGP (("I cannot find the corresponding URL.\n"));
       /* Parse the HTML file...  */
-      urls = get_urls_html (html->string, urlhtml ? urlhtml->url : NULL, 1);
+      urls = get_urls_html (html->string, urlhtml ? urlhtml->url : NULL, 1,
+			    FALSE);
       if (!urls)
 	continue;
       for (l1 = urls; l1; l1 = l1->next)
@@ -628,7 +653,7 @@ parse_robots (const char *robots_filename)
   char **entries;
   char *line, *cmd, *str, *p;
   char *base_version, *version;
-  int len, num, i;
+  int num, i;
   int wget_matched;		/* is the part meant for Wget?  */
 
   entries = NULL;
@@ -639,23 +664,23 @@ parse_robots (const char *robots_filename)
     return NULL;
 
   /* Kill version number.  */
-    if (opt.useragent)
-      {
-	STRDUP_ALLOCA (base_version, opt.useragent);
-	STRDUP_ALLOCA (version, opt.useragent);
-      }
-    else
-      {
-	int len = 10 + strlen (version_string);
-	base_version = (char *)alloca (len);
-	sprintf (base_version, "Wget/%s", version_string);
-	version = (char *)alloca (len);
-	sprintf (version, "Wget/%s", version_string);
-      }
+  if (opt.useragent)
+    {
+      STRDUP_ALLOCA (base_version, opt.useragent);
+      STRDUP_ALLOCA (version, opt.useragent);
+    }
+  else
+    {
+      int len = 10 + strlen (version_string);
+      base_version = (char *)alloca (len);
+      sprintf (base_version, "Wget/%s", version_string);
+      version = (char *)alloca (len);
+      sprintf (version, "Wget/%s", version_string);
+    }
   for (p = version; *p; p++)
-    *p = tolower (*p);
+    *p = TOLOWER (*p);
   for (p = base_version; *p && *p != '/'; p++)
-    *p = tolower (*p);
+    *p = TOLOWER (*p);
   *p = '\0';
 
   /* Setting this to 1 means that Wget considers itself under
@@ -680,10 +705,12 @@ parse_robots (const char *robots_filename)
   wget_matched = 1;
   while ((line = read_whole_line (fp)))
     {
-      len = strlen (line);
-      /* Destroy <CR> if there is one.  */
+      int len = strlen (line);
+      /* Destroy <CR><LF> if present.  */
+      if (len && line[len - 1] == '\n')
+	line[--len] = '\0';
       if (len && line[len - 1] == '\r')
-	line[len - 1] = '\0';
+	line[--len] = '\0';
       /* According to specifications, optional space may be at the
 	 end...  */
       DEBUGP (("Line: %s\n", line));
@@ -707,13 +734,13 @@ parse_robots (const char *robots_filename)
       *str++ = '\0';
       /* Look for the string beginning...  */
       for (; *str && ISSPACE (*str); str++);
-      /* Look for comments and kill them off.  */
+      /* Look for comments or trailing spaces and kill them off.  */
       for (p = str; *p; p++)
-	if (*p && ISSPACE (*p) && *(p + 1) == '#')
+	if (*p && ISSPACE (*p) && ((*(p + 1) == '#') || (*(p + 1) == '\0')))
 	  {
-	    /* We have found a shell-style comment `<sp>+ #'.  Now
-	       rewind to the beginning of the spaces and place '\0'
-	       there.  */
+	    /* We have found either a shell-style comment `<sp>+#' or some
+               trailing spaces.  Now rewind to the beginning of the spaces
+               and place '\0' there.  */
 	    while (p > str && ISSPACE (*p))
 	      --p;
 	    if (p == str)
@@ -727,7 +754,7 @@ parse_robots (const char *robots_filename)
 	  int match = 0;
 	  /* Lowercase the agent string.  */
 	  for (p = str; *p; p++)
-	    *p = tolower (*p);
+	    *p = TOLOWER (*p);
 	  /* If the string is `*', it matches.  */
 	  if (*str == '*' && !*(str + 1))
 	    match = 1;
@@ -797,19 +824,19 @@ parse_robots (const char *robots_filename)
 /* May the URL url be loaded according to disallowing rules stored in
    forbidden?  */
 static int
-robots_match (struct urlinfo *u, char **forbidden)
+robots_match (struct urlinfo *u, char **fb)
 {
   int l;
 
-  if (!forbidden)
+  if (!fb)
     return 1;
   DEBUGP (("Matching %s against: ", u->path));
-  for (; *forbidden; forbidden++)
+  for (; *fb; fb++)
     {
-      DEBUGP (("%s ", *forbidden));
-      l = strlen (*forbidden);
-      /* If dir is forbidden, we may not load the file.  */
-      if (strncmp (u->path, *forbidden, l) == 0)
+      DEBUGP (("%s ", *fb));
+      l = strlen (*fb);
+      /* If dir is fb, we may not load the file.  */
+      if (strncmp (u->path, *fb, l) == 0)
 	{
 	  DEBUGP (("matched.\n"));
 	  return 0; /* Matches, i.e. does not load...  */
