@@ -1,5 +1,5 @@
 /* Dirty system-dependent hacks.
-   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
 
 This file is part of Wget.
 
@@ -45,15 +45,15 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <mswindows.h>
 #endif /* WINDOWS */
 
-/* Allegedly needed for compilation under OS/2: */
-#ifdef EMXOS2
+/* Needed for compilation under OS/2: */
+#ifdef __EMX__
 #ifndef S_ISLNK
 # define S_ISLNK(m) 0
 #endif
 #ifndef lstat
 # define lstat stat
 #endif
-#endif /* EMXOS2 */
+#endif /* __EMX__ */
 
 /* Reportedly, stat() macros are broken on some old systems.  Those
    systems will have to fend for themselves, as I will not introduce
@@ -97,8 +97,22 @@ do {						\
   DEBUGP (("Closing fd %d\n", x));		\
 } while (0)
 
+/* Define a "very long" type useful for storing large non-negative
+   integers, e.g. the total number of bytes downloaded.  This needs to
+   be an integral type at least 64 bits wide.  On the machines where
+   `long' is 64-bit, we use long.  Otherwise, we check whether `long
+   long' is available and if yes, use that.  Otherwise, we give up and
+   just use `long'.  */
+#if (SIZEOF_LONG >= 8) || !defined(HAVE_LONG_LONG)
+# define VERY_LONG_TYPE   unsigned long
+# define VERY_LONG_FORMAT "%lu"
+#else  /* long is smaller than 8 bytes, but long long is available. */
+# define VERY_LONG_TYPE   unsigned long long
+# define VERY_LONG_FORMAT "%llu"
+#endif /* use long long */
+
 /* OK, now define a decent interface to ctype macros.  The regular
-   ones misfire when you feed them chars >= 127, as they understand
+   ones misfire when you feed them chars > 127, as they understand
    them as "negative", which results in out-of-bound access at
    table-lookup, yielding random results.  This is, of course, totally
    bogus.  One way to "solve" this is to use `unsigned char'
@@ -110,9 +124,12 @@ do {						\
    versions are used consistently across the code.  */
 #define ISASCII(x)  isascii ((unsigned char)(x))
 #define ISALPHA(x)  isalpha ((unsigned char)(x))
+#define ISALNUM(x)  isalnum ((unsigned char)(x))
 #define ISSPACE(x)  isspace ((unsigned char)(x))
 #define ISDIGIT(x)  isdigit ((unsigned char)(x))
 #define ISXDIGIT(x) isxdigit ((unsigned char)(x))
+#define TOUPPER(x)  toupper ((unsigned char)(x))
+#define TOLOWER(x)  tolower ((unsigned char)(x))
 
 /* Defined in cmpt.c: */
 #ifndef HAVE_STRERROR
@@ -130,6 +147,9 @@ char *strstr ();
 #ifndef HAVE_STRPTIME
 char *strptime ();
 #endif
+#ifndef HAVE_VSNPRINTF
+int vsnprintf ();
+#endif
 
 /* SunOS brain damage -- for some reason, SunOS header files fail to
    declare the functions below, which causes all kinds of problems
